<?php
namespace FiloBlu\PrivacyPolicy\Model\Config\Backend;

use Magento\Customer\Model\Customer;
use Magento\Eav\Model\AttributeRepository;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Value;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;

class PrivacyStoreVersion extends Value
{
    /**
     * @var AttributeRepository
     */
    private $attributeRepository;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    /**
     * PrivacyStoreVersion constructor.
     * @param Context $context
     * @param Registry $registry
     * @param ScopeConfigInterface $config
     * @param TypeListInterface $cacheTypeList
     * @param AttributeRepository $attributeRepository
     * @param AbstractResource|null $resource
     * @param AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        Context $context,
        Registry $registry,
        ScopeConfigInterface $config,
        TypeListInterface $cacheTypeList,
        AttributeRepository $attributeRepository,
        AbstractResource $resource = null,
        AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        parent::__construct($context, $registry, $config, $cacheTypeList, $resource, $resourceCollection, $data);
        $this->attributeRepository = $attributeRepository;
        $this->logger = $context->getLogger();
    }

    /**
     * @return Value
     * @throws \Magento\Framework\Exception\ValidatorException
     */
    public function beforeSave()
    {
        try {
            $value = $this->getValue();
            $this->updateAttributeDefaultValue($value);
            return parent::beforeSave();
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
            throw new \Magento\Framework\Exception\ValidatorException(__($e->getMessage()));
        } catch (\Throwable $t) {
            $this->logger->critical($t->getMessage());
            throw new \Magento\Framework\Exception\ValidatorException(__($t->getMessage()));
        }
    }

    /**
     * @param $value
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Exception\StateException
     */
    protected function updateAttributeDefaultValue($value)
    {
        $privacyVersion = $this->attributeRepository->get(Customer::ENTITY, \FiloBlu\PrivacyPolicy\Setup\InstallData::PRIVACY_VERSION_CODE);
        $defaultValue = $privacyVersion->getDefaultValue() ?? "";
        if ($value != $defaultValue) {
            $privacyVersion->setDefaultValue($value);
            $this->attributeRepository->save($privacyVersion);
        }
    }
}
